@@ -241,47 +241,81 @@ class Index(typing.NamedTuple):
 
 
 STANDARD_INDEX = Index("standard", None, "pk", "sk")
+PARTITION_G1_INDEX = Index("partition_g1", "partition_g1", "pk", "g1k")
+PARTITION_G2_INDEX = Index("partition_g2", "partition_g2", "pk", "g2k")
+PARTITION_G3_INDEX = Index("partition_g3", "partition_g3", "pk", "g3k")
+
 INVERTED_INDEX = Index("inverted", "inverted", "sk", "pk")
+SORT_G1_INDEX = Index("sort_g1", "sort_g1", "sk", "g1k")
+SORT_G2_INDEX = Index("sort_g2", "sort_g2", "sk", "g2k")
+SORT_G3_INDEX = Index("sort_g3", "sort_g3", "sk", "g3k")
 
 G1_PARTITION_INDEX = Index("g1_partition", "g1_partition", "g1k", "pk")
-PARTITION_G1_INDEX = Index("partition_g1", "partition_g1", "pk", "g1k")
 G1_SORT_INDEX = Index("g1_sort", "g1_sort", "g1k", "sk")
-SORT_G1_INDEX = Index("sort_g1", "sort_g1", "sk", "g1k")
+G1_G2_INDEX = Index("g1_g2", "g1_g2", "g1", "g2")
+G1_G3_INDEX = Index("g1_g3", "g1_g3", "g1", "g3")
 
 G2_PARTITION_INDEX = Index("g2_partition", "g2_partition", "g2k", "pk")
-PARTITION_G2_INDEX = Index("partition_g2", "partition_g2", "pk", "g2k")
 G2_SORT_INDEX = Index("g2_sort", "g2_sort", "g2k", "sk")
-SORT_G2_INDEX = Index("sort_g2", "sort_g2", "sk", "g2k")
+G2_G1_INDEX = Index("g2_g1", "g2_g1", "g2", "g1")
+G2_G3_INDEX = Index("g2_g3", "g2_g3", "g2", "g3")
+
+G3_PARTITION_INDEX = Index("g3_partition", "g3_partition", "g3k", "pk")
+G3_SORT_INDEX = Index("g3_sort", "g3_sort", "g3k", "sk")
+G3_G1_INDEX = Index("g3_g1", "g3_g1", "g3", "g1")
+G3_G2_INDEX = Index("g3_g3", "g3_g3", "g3", "g3")
 
 
 class Indexes:
     """Possible indexes on tables, including GSI indexes."""
 
     STANDARD = STANDARD_INDEX
+    PARTITION_G1 = PARTITION_G1_INDEX
+    PARTITION_G2 = PARTITION_G2_INDEX
+    PARTITION_G3 = PARTITION_G3_INDEX
+
     INVERTED = INVERTED_INDEX
+    SORT_G1 = SORT_G1_INDEX
+    SORT_G2 = SORT_G2_INDEX
+    SORT_G3 = SORT_G3_INDEX
 
     G1_PARTITION = G1_PARTITION_INDEX
-    PARTITION_G1 = PARTITION_G1_INDEX
     G1_SORT = G1_SORT_INDEX
-    SORT_G1 = SORT_G1_INDEX
+    G1_G2 = G1_G2_INDEX
+    G1_G3 = G1_G3_INDEX
 
     G2_PARTITION = G2_PARTITION_INDEX
-    PARTITION_G2 = PARTITION_G2_INDEX
     G2_SORT = G2_SORT_INDEX
-    SORT_G2 = SORT_G2_INDEX
+    G2_G1 = G2_G1_INDEX
+    G2_G3 = G2_G3_INDEX
+
+    G3_PARTITION = G3_PARTITION_INDEX
+    G3_SORT = G3_SORT_INDEX
+    G3_G1 = G3_G1_INDEX
+    G3_G2 = G3_G2_INDEX
 
 
 INDEXES_LIST = [
     STANDARD_INDEX,
-    INVERTED_INDEX,
-    G1_PARTITION_INDEX,
     PARTITION_G1_INDEX,
-    G1_SORT_INDEX,
-    SORT_G1_INDEX,
-    G2_PARTITION_INDEX,
     PARTITION_G2_INDEX,
-    G2_SORT_INDEX,
+    PARTITION_G3_INDEX,
+    INVERTED_INDEX,
+    SORT_G1_INDEX,
     SORT_G2_INDEX,
+    SORT_G3_INDEX,
+    G1_PARTITION_INDEX,
+    G1_SORT_INDEX,
+    G1_G2_INDEX,
+    G1_G3_INDEX,
+    G2_PARTITION_INDEX,
+    G2_SORT_INDEX,
+    G2_G1_INDEX,
+    G2_G3_INDEX,
+    G3_PARTITION_INDEX,
+    G3_SORT_INDEX,
+    G3_G1_INDEX,
+    G3_G2_INDEX,
 ]
 
 
@@ -299,7 +333,7 @@ class ColumnType(typing.Protocol):
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -316,7 +350,7 @@ class Column:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -331,7 +365,7 @@ class StringColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -348,7 +382,7 @@ class BooleanColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -365,7 +399,7 @@ class BinarySetColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -384,7 +418,7 @@ class BytesColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -401,7 +435,7 @@ class DateColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -418,7 +452,7 @@ class DatetimeColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -435,7 +469,7 @@ class ListColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -452,7 +486,7 @@ class FloatColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -469,7 +503,7 @@ class FloatSetColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -486,7 +520,7 @@ class IntegerColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -503,7 +537,7 @@ class IntegerSetColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -523,7 +557,7 @@ class MapColumn:
     children: typing.Tuple[Column, ...]
     #: Name of the key in the table
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -540,7 +574,7 @@ class StringSetColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -559,7 +593,7 @@ class TimestampColumn:
     #: Name of the key in the table. If not specified the name will match
     #: the name value. Use this for additional keys.
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -577,7 +611,7 @@ class IndexedColumn:
     value_prefix: str
     #: Name of the key in the table
     key: typing.Optional[str] = None
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -595,7 +629,7 @@ class PartitionColumn:
     value_prefix: str
     #: Name of the key in the table
     key: str = dataclasses.field(default="pk", init=False)
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -613,7 +647,7 @@ class SortColumn:
     value_prefix: str
     #: Name of the key in the table
     key: str = dataclasses.field(default="sk", init=False)
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -649,7 +683,25 @@ class GlobalSecondColumn:
     value_prefix: str
     #: Name of the key in the table
     key: str = dataclasses.field(default="g2k", init=False)
-    #: Whether or not the field is computed instead of stored directly in the
+    #: Whether the field is computed instead of stored directly in the
+    #: model. Computed fields will be written to the database record but not
+    #: loaded back into the model.
+    computed: bool = False
+    #: Data type for the column. Will always be of type String.
+    data_type: DynamoType = dataclasses.field(default=DynamoTypes.STRING, init=False)
+
+
+@dataclasses.dataclass(frozen=True)
+class GlobalThirdColumn:
+    """Data structure for a quaternary key column within a DynamoDB table."""
+
+    #: Name of the column.
+    name: str
+    #: Value prefix within the table records
+    value_prefix: str
+    #: Name of the key in the table
+    key: str = dataclasses.field(default="g3k", init=False)
+    #: Whether the field is computed instead of stored directly in the
     #: model. Computed fields will be written to the database record but not
     #: loaded back into the model.
     computed: bool = False
@@ -686,25 +738,42 @@ class Schema:
         :param row:
             A DynamoDB row record.
         """
+        pk_name = self._is_partition_key_prefix_match(row)
+        if pk_name is None:
+            return False
+
+        sk_prefix_match, sk_name = self._is_sort_key_prefix_match(row)
+        if not sk_prefix_match:
+            return False
+
+        return self._is_exact_match(row, pk_name, sk_name)
+
+    def _is_partition_key_prefix_match(self, row: dict) -> typing.Optional[str]:
+        """Match the partition key with the row and return the comparison."""
         partition_key_name = self.partition_key.key or "pk"
         partition_key = row[partition_key_name]["S"]
-        if not partition_key.startswith(self.partition_key.value_prefix):
-            return False
+        is_match = partition_key.startswith(self.partition_key.value_prefix)
+        return partition_key_name if is_match else None
 
-        sort_key_name = self.sort_key.key if self.sort_key else None
-        sort_key_name = sort_key_name or "sk"
+    def _is_sort_key_prefix_match(self, row: dict) -> typing.Tuple[bool, str]:
+        """Match the sort key with the row and return the comparison."""
+        sort_key_name = getattr(self.sort_key, "key", None) or "sk"
         sort_key = row.get(sort_key_name, {}).get("S")
-        if sort_key is not None and not self.sort_key:
-            return False
+        value_prefix = getattr(self.sort_key, "value_prefix", None)
+        is_mismatch = (
+            (sort_key is None and self.sort_key)
+            or (sort_key is not None and not self.sort_key)
+            or (sort_key and value_prefix and not sort_key.startswith(value_prefix))
+        )
+        return not is_mismatch, sort_key_name
 
-        if self.sort_key and sort_key is None:
-            return False
-
-        value_prefix = self.sort_key.value_prefix if self.sort_key else None
-        if sort_key and value_prefix and not sort_key.startswith(value_prefix):
-            return False
-
-        # Only match if all keys in the row are present in the schema.
+    def _is_exact_match(
+        self,
+        row: dict,
+        partition_key_name: str,
+        sort_key_name: str,
+    ) -> bool:
+        """Only match if all keys in the row are present in the schema."""
         key_names = (partition_key_name, sort_key_name)
         schema_keys = {c.key or c.name for c in self.all_columns}
         row_keys = {k for k in row.keys() if k not in key_names}
